@@ -25,9 +25,9 @@ from Products.CMFCore.exceptions import BadRequest
 from Products.PluggableAuthService.PluggableAuthService \
     import addPluggableAuthService
 from Products.PlonePAS.Extensions.Install import activatePluginInterfaces
-from Products.PlonePAS.Extensions.Install import challenge_chooser_setup
-from Products.PlonePAS.Extensions.Install import registerPluginTypes
-from Products.PlonePAS.Extensions.Install import setupPlugins
+from Products.PlonePAS.setuphandlers import challenge_chooser_setup
+from Products.PlonePAS.setuphandlers import registerPluginTypes
+from Products.PlonePAS.setuphandlers import setupPlugins
 
 from Products.PloneSurvey import PloneSurveyMessageFactory as _
 from Products.PloneSurvey.config import PROJECTNAME
@@ -286,6 +286,16 @@ class Survey(ATCTOrderedFolder):
                 return page()
         return self.exitSurvey()
 
+    security.declareProtected(permissions.View, 'hasMorePages')
+
+    def hasMorePages(self):
+        """Return True if survey has more pages to display"""
+        pages = self.getFolderContents(
+            contentFilter={'portal_type': 'Sub Survey', }, full_objects=True)
+        if not pages:
+            return False
+        return True
+
     security.declareProtected(permissions.View, 'exitSurvey')
 
     def exitSurvey(self):
@@ -377,7 +387,6 @@ class Survey(ATCTOrderedFolder):
         # expires = (DateTime() + 365).toZone('GMT').rfc822()
         # cookie expires in 1 year (365 days)
         response.setCookie(survey_cookie, user_id, path='/')
-        self.addRespondent(user_id)
         return user_id
 
     security.declareProtected(permissions.View, 'getAnonymousId')
@@ -394,30 +403,18 @@ class Survey(ATCTOrderedFolder):
             return anon_id + str(DateTime())
         elif portal_membership.isAnonymousUser():
             return self.REQUEST.RESPONSE.redirect(
-                self.portal_url()+'/login_form?came_from='+self.absolute_url())
+                self.portal_url() + '/login_form?came_from=' + self.absolute_url())
         return portal_membership.getAuthenticatedMember().getId()
 
     security.declareProtected(permissions.View, 'getRemoteIp')
 
     def getRemoteIp(self, request=None):
         """returns the ip address of the survey respondent"""
-        # XXX put in placeholder for working out the ip address
         if self.getConfidential():
             return
         if request is None:
             request = getattr(self, 'REQUEST', None)
         return request.getClientAddr()
-        ip_address = self.REQUEST.environ['REMOTE_ADDR']
-        if ip_address == "127.0.0.1":
-            try:
-                ip_address = self.REQUEST.environ['HTTP_X_FORWARDED_FOR']
-            except KeyError:
-                # might be using it on a localhost
-                return
-        return ip_address
-        # return self.REQUEST.getClientAddr()
-        # return self.REQUEST['REMOTE_ADDR']
-        # return self.REQUEST['HTTP_X_FORWARDED_FOR']
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'getRespondentsDetails')
@@ -478,18 +475,6 @@ class Survey(ATCTOrderedFolder):
             ip_address=self.getRemoteIp(),
             end='')
 
-    security.declareProtected(permissions.ModifyPortalContent,
-                              'getRespondents')
-
-    def getRespondents(self):
-        """Return a list of respondents"""
-        questions = self.getAllQuestionsInOrder()
-        users = {}
-        for question in questions:
-            for user in question.answers.keys():
-                users[user] = 1
-        return users.keys()
-
     security.declareProtected(ViewSurveyResults, 'getRespondentFullName')
 
     def getRespondentFullName(self, userid):
@@ -572,12 +557,12 @@ class Survey(ATCTOrderedFolder):
         mFrom = properties.email_from_address
         mSubj = translate(_(
             '[${survey_title}] New survey submitted',
-            mapping={'survey_title': self.Title()}),
+            mapping={'survey_title': safe_unicode(self.Title())}),
             context=self.REQUEST)
         message = []
         message.append(translate(_(
             'Survey ${survey_title}',
-            mapping={'survey_title': self.Title()}),
+            mapping={'survey_title': safe_unicode(self.Title())}),
             context=self.REQUEST))
         message.append(translate(_(
             'has been completed by user: ${userid}',
@@ -587,7 +572,7 @@ class Survey(ATCTOrderedFolder):
                        '/@@Products.PloneSurvey.survey_view_results')
         mMsg = '\n\n'.join(message)
         try:
-            self.MailHost.send(mMsg.encode('utf-8'), mTo, mFrom, mSubj)
+            self.MailHost.send(mMsg, mTo, mFrom, mSubj, charset='utf-8')
         except ConflictError:
             raise
         except:
@@ -723,15 +708,19 @@ class Survey(ATCTOrderedFolder):
             '<a href="' + survey_url + '">' + self.Title() + '</a>')
         mail_text = self.survey_send_invite_template(
             user=user,
-            recipient=user.getId(),
-            email_from_name=email_from_name,
-            email_from_address=email_from_address,
             email_body=email_body,
-            subject="Survey %s" % self.title_or_id())
+            subject=self.title_or_id())
         host = self.MailHost
         site_props = portal_properties.site_properties
-        mail_text = mail_text.encode(site_props.default_charset or 'utf-8')
-        host.send(mail_text)
+        charset = site_props.default_charset or 'utf-8'
+        mail_text = safe_unicode(mail_text)
+        host.send(mail_text,
+                  mto=user.getId(),
+                  mfrom=u"{} <{}>".format(
+                        safe_unicode(email_from_name), email_from_address),
+                  subject=safe_unicode(self.title_or_id()),
+                  charset=charset,
+                  msg_type='text/html')
         self.registerRespondentSent(email_address)
 
     security.declareProtected(permissions.ModifyPortalContent,
@@ -743,7 +732,7 @@ class Survey(ATCTOrderedFolder):
         if use_transactions:
             transaction.abort()
         respondents = self.acl_users.getUsers()
-        already_completed = self.getRespondents()
+        already_completed = self.getRespondentsList()
         for respondent in respondents:
             if use_transactions:
                 transaction.get()
@@ -820,7 +809,7 @@ class Survey(ATCTOrderedFolder):
         sheet.writerow(('user',) + tuple(q.Title()
                        for q in questions) + ('completed',))
 
-        for user in self.getRespondents():
+        for user in self.getRespondentsList():
             if self.getConfidential():
                 row = ['Anonymous']
             else:
@@ -829,14 +818,19 @@ class Survey(ATCTOrderedFolder):
                 answer = question.getAnswerFor(user) or ''
                 # handle there being no answer (e.g branched question)
                 if answer:
-                    if not (isinstance(answer, str) or isinstance(answer, int)):
+                    if not (isinstance(answer, str) or \
+                            isinstance(answer, unicode) or \
+                            isinstance(answer, int)):
                         # It's a sequence, filter out empty values
                         answer = ', '.join(filter(None, answer))
                 row.append(answer.replace('"', "'").replace('\r\n', ' '))
 
             row.append(self.checkCompletedFor(user) and
                        'Completed' or 'Not Completed')
-
+            for i, col in enumerate(row):
+                if isinstance(col, unicode):
+                    col = col.encode('utf8')
+                row[i] = col
             sheet.writerow(row)
 
         return data.getvalue()
@@ -886,7 +880,7 @@ class Survey(ATCTOrderedFolder):
             ignore_meta_types=['SurveyMatrix', ])
         sheet.writerow(('user',) + tuple(q.Title()
                        for q in questions) + ('completed',))
-        for user in self.getRespondents():
+        for user in self.getRespondentsList():
             if self.getConfidential():
                 row = ['Anonymous']
             else:
@@ -909,7 +903,7 @@ class Survey(ATCTOrderedFolder):
                                 answer += '1;'
                             else:
                                 answer += '0;'
-                        answer = '"' + answer[0:len(answer)-1] + '"'
+                        answer = '"' + answer[0:len(answer) - 1] + '"'
                     elif answerList:
                         answer = '"' + answerList + '"'
                     else:
@@ -998,7 +992,7 @@ class Survey(ATCTOrderedFolder):
                 options_row.append(options[i])
         sheet.writerow(row)
         sheet.writerow(options_row)
-        for user in self.getRespondents():
+        for user in self.getRespondentsList():
             if self.getConfidential():
                 row = ['Anonymous']
             else:
